import { NetworkStatus } from "apollo-client";
import { ApolloProviderProviderConsumer } from "../../utils/provider";
export class ApolloQueryComponent {
    componentWillLoad() {
        this.startSubscription();
    }
    onPropsChange() {
        this.stopSubscription();
        this.startSubscription();
    }
    componentDidUnload() {
        this.stopSubscription();
    }
    startSubscription() {
        if (this.client) {
            const observable = this.client.watchQuery(Object.assign({ query: this.query, variables: this.variables }, this.options));
            this.result = {
                data: undefined,
                loading: true,
                error: undefined,
                variables: this.variables,
                networkStatus: undefined,
                refetch: observable.refetch.bind(observable),
                fetchMore: observable.fetchMore.bind(observable),
                startPolling: observable.startPolling.bind(observable),
                stopPolling: observable.stopPolling.bind(observable),
                subscribeToMore: observable.subscribeToMore.bind(observable),
                updateQuery: observable.updateQuery.bind(observable),
                client: this.client,
            };
            this._subscription = observable.subscribe(queryResult => {
                const { data, loading, errors, networkStatus, } = queryResult;
                this.result = {
                    data,
                    loading,
                    error: {
                        graphQLErrors: errors,
                        networkError: undefined,
                        message: errors && errors[0].message,
                        name: errors && errors[0].name,
                        extraInfo: errors && errors[0],
                    },
                    variables: this.variables,
                    networkStatus,
                    refetch: observable.refetch.bind(observable),
                    fetchMore: observable.fetchMore.bind(observable),
                    startPolling: observable.startPolling.bind(observable),
                    stopPolling: observable.stopPolling.bind(observable),
                    subscribeToMore: observable.subscribeToMore.bind(observable),
                    updateQuery: observable.updateQuery.bind(observable),
                    client: this.client,
                };
                this.resultEventEmitter.emit(this.result);
            }, error => {
                this.result = {
                    data: undefined,
                    loading: false,
                    error,
                    variables: this.variables,
                    networkStatus: NetworkStatus.error,
                    refetch: observable.refetch.bind(observable),
                    fetchMore: observable.fetchMore.bind(observable),
                    startPolling: observable.startPolling.bind(observable),
                    stopPolling: observable.stopPolling.bind(observable),
                    subscribeToMore: observable.subscribeToMore.bind(observable),
                    updateQuery: observable.updateQuery.bind(observable),
                    client: this.client,
                };
                this.resultEventEmitter.emit(this.result);
            });
            this.readyEventEmitter.emit(this.result);
        }
        else {
            throw new Error('You should wrap your parent component with apollo-provider custom element or ApolloProvider functional component');
        }
    }
    stopSubscription() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
    }
    render() {
        return this.renderer && this.renderer(this.result);
    }
    static get is() { return "apollo-query"; }
    static get properties() { return {
        "client": {
            "type": "Any",
            "attr": "client",
            "watchCallbacks": ["onPropsChange"]
        },
        "el": {
            "elementRef": true
        },
        "options": {
            "type": "Any",
            "attr": "options",
            "watchCallbacks": ["onPropsChange"]
        },
        "query": {
            "type": "Any",
            "attr": "query",
            "watchCallbacks": ["onPropsChange"]
        },
        "renderer": {
            "type": "Any",
            "attr": "renderer",
            "watchCallbacks": ["onPropsChange"]
        },
        "result": {
            "state": true
        },
        "variables": {
            "type": "Any",
            "attr": "variables",
            "watchCallbacks": ["onPropsChange"]
        }
    }; }
    static get events() { return [{
            "name": "ready",
            "method": "readyEventEmitter",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "result",
            "method": "resultEventEmitter",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}
ApolloProviderProviderConsumer.injectProps(ApolloQueryComponent, ['client']);
